# Add descriptions here

import logging
import hashlib

log = logging.getLogger(__name__)

# Look at the requests module
try:
    import requests
    from requests.exceptions import HTTPError
except ModuleNotFoundError:
    print("There is some issue with the module requests")
    raise


def check_pwd(pwd):
    """Check if the given passwpord has been breached online.
    The password is basically checked in the pwnedpasswords
    website.

    Parameters
    ----------
        pwd : str
            Password

    Returns
    -------
        tuple:
            hashpwd password in has
            count the number of appearance
    """
    # Convert password into hash format
    hashpwd = hashlib.sha1(pwd.encode("utf-8")).hexdigest().upper()
    # Slice the password into two parts
    head, tail = hashpwd[:5], hashpwd[5:]
    # Define the url appended with the head
    url = "https://api.pwnedpasswords.com/range/" + head
    # Check the url status
    try:
        log.info("Accessing https://api.pwnedpasswords.com/range/")
        response = requests.get(url)
        response.raise_for_status()
    except HTTPError as http_error:
        log.warning(f"HTTP error occurred: {http_error}")
    except Exception as error:
        log.warning(f"Other error occurred: {error}")
    else:
        log.info("Success!")
    # Format all the corresponding entries
    hashes = (line.split(":") for line in response.text.splitlines())
    # Count the number of times the pwd appears
    count = next((int(count) for t, count in hashes if t == tail), 0)
    # Return the hashed pwd and count
    return hashpwd, count



"""
This file contains the current state of packaging in Python using
Distribution Utilities (Distutils) and its extension from the end
user'point-of-view.

Documentation:
https://the-hitchhikers-guide-to-packaging.readthedocs.io/en/latest/introduction.html
"""

import os
import re
import sys
from setuptools import setup
from setuptools import find_packages

PACKAGE = "checkpwd"

# Used for pytest and code coverage
TESTS_REQUIEREMENTS = ["pytest", "pytest-cov"]
# Depending on the documents more dependencies can be added
DOCS_REQUIEREMENTS = ["recommonmark", "sphinx_rtd_theme", "sphinxcontrib-bibtex"]
# Dependencies for the packages
PACKAGE_REQUIEREMENTS = ["requests"]


# Read through Readme
try:
    this_directory = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as f:
        long_description = f.read()
except IOError:
    print("Read me file not found.")


def get_version():
    """Gets the version from the package's __init__ file
    if there is some problem, this fails.
    """
    VERSIONFILE = os.path.join("src", PACKAGE, "__init__.py")
    initfile_lines = open(VERSIONFILE, "rt").readlines()
    VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
    for line in initfile_lines:
        mo = re.search(VSRE, line, re.M)
        if mo:
            return mo.group(1)


setup(
    name=PACKAGE,
    version=get_version(),
    description="Check data breaches",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Tanjona R. Rabemananjara",
    author_email="tanjona.rabemananjara@unimi.it",
    url="https://github.com/Radonirinaunimi/pwnd-check/",
    install_requires=PACKAGE_REQUIEREMENTS,
    extras_require={"docs": DOCS_REQUIEREMENTS, "tests": TESTS_REQUIEREMENTS},
    entry_points={"console_scripts": ["checkpwd = checkpwd.run:main", ]},
    package_dir={"": "src"},
    packages=find_packages("src"),
    package_data = {'' : ['logo/logo.png'], },
    zip_safe=False,
    classifiers=[
        "Operating System :: Unix",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Physics",
    ],
    setup_requires=['wheel'],
    python_requires='>=3.7'
)



# Main file for the PwnedCheck distribution.
# This file retrieves the password, calls the module check_pwd,
# and check if the password has been breached by checking it with
# the database in the following website
# https://haveibeenpwned.com/Passwords

import sys
import logging
from getpass import getpass
from checkpwd.check_pwd import check_pwd

logging.basicConfig(level=logging.INFO, format="[%(levelname)s] %(message)s")
log = logging.getLogger(__name__)


def splash():
    """Splash `pwnedcheck` logo and information."""
    print(" _____________________________________________________ ")
    print("|   ____ _               _    ____              _     |")
    print("|  / ___| |__   ___  ___| | _|  _ \__      ____| |    |")  
    print("| | |   | '_ \ / _ \/ __| |/ / |_) \ \ /\ / / _` |    |")
    print("| | |___| | | |  __/ (__|   <|  __/ \ V  V / (_| |    |")
    print("|  \____|_| |_|\___|\___|_|\_\_|     \_/\_/ \__,_|    |")
    print("|                                                     |")
    print("| Author: Tanjona R. Rabemananjara                    |")
    print("| URL: https://radonirinaunimi.github.io/pwnd-check/  |")
    print("|_____________________________________________________|")


def main():
    """Function that fetchs the password given by the user from the command
    line using `getpass`. The password is then checked on `HaveIBeenPwned`.
    """
    splash()
    print("\nEnter your password below.")
    pwd = getpass()
    try:
        # Check the pwd and add the values to some variables
        hashed_pwd, nb_match = check_pwd(pwd)
        # Print the result
        if nb_match:
            print(f"The password occurs {nb_match} times (hash: {hashed_pwd})")
        else:
            print("Your password was not found")
    except UnicodeError:
        errormsg = sys.exc_info()[1]
        log.warning(f"Your password could not be checked: {errormsg}")
